# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.2

import enum

import winrt.system
from winrt import _winrt_windows_devices_gpio

__all__ = [
    "GpioChangePolarity",
    "GpioOpenStatus",
    "GpioPinDriveMode",
    "GpioPinEdge",
    "GpioPinValue",
    "GpioSharingMode",
    "GpioChangeCount",
    "GpioChangeRecord",
    "GpioChangeCounter",
    "GpioChangeReader",
    "GpioController",
    "GpioPin",
    "GpioPinValueChangedEventArgs",
]

class GpioChangePolarity(enum.IntEnum):
    FALLING = 0
    RISING = 1
    BOTH = 2

class GpioOpenStatus(enum.IntEnum):
    PIN_OPENED = 0
    PIN_UNAVAILABLE = 1
    SHARING_VIOLATION = 2
    MUXING_CONFLICT = 3
    UNKNOWN_ERROR = 4

class GpioPinDriveMode(enum.IntEnum):
    INPUT = 0
    OUTPUT = 1
    INPUT_PULL_UP = 2
    INPUT_PULL_DOWN = 3
    OUTPUT_OPEN_DRAIN = 4
    OUTPUT_OPEN_DRAIN_PULL_UP = 5
    OUTPUT_OPEN_SOURCE = 6
    OUTPUT_OPEN_SOURCE_PULL_DOWN = 7

class GpioPinEdge(enum.IntEnum):
    FALLING_EDGE = 0
    RISING_EDGE = 1

class GpioPinValue(enum.IntEnum):
    LOW = 0
    HIGH = 1

class GpioSharingMode(enum.IntEnum):
    EXCLUSIVE = 0
    SHARED_READ_ONLY = 1

GpioChangeCount = _winrt_windows_devices_gpio.GpioChangeCount
GpioChangeRecord = _winrt_windows_devices_gpio.GpioChangeRecord
GpioChangeCounter = _winrt_windows_devices_gpio.GpioChangeCounter
GpioChangeReader = _winrt_windows_devices_gpio.GpioChangeReader
GpioController = _winrt_windows_devices_gpio.GpioController
GpioPin = _winrt_windows_devices_gpio.GpioPin
GpioPinValueChangedEventArgs = _winrt_windows_devices_gpio.GpioPinValueChangedEventArgs
